import { world, system, EntityEquippableComponent, EquipmentSlot, ItemStack } from "@minecraft/server";
import {  custom_bow, custom_trident, item_animations, ambar_totem, Durability } from './custom_component.js';
import { addDamage, removeEffect, addEffect, getScore, addScore, setScore } from './functions.js';
import {shields_loop} from './shield.js'
import { break3x3Area } from './common/blockBreaker';
import { HAMMER_BLOCK_TYPES } from './tools/hammerTypes';
import { DRILL_BLOCK_TYPES } from './tools/drillTypes';

const tags= ["topacio", "jade", "ambar", "uranium", "onix", "ruby", "sapphire", "flourite", "platino"]

world.beforeEvents.playerBreakBlock.subscribe(({ block, player }) => {
   try{
      const component = player.getComponent(EntityEquippableComponent.componentId);
      const tool = component.getEquipment(EquipmentSlot.Mainhand);
      if (tool && !player.isSneaking) {
         const enchantments = tool.getComponent('enchantable');
         const fortuneLevel = enchantments.hasEnchantment('fortune') ? enchantments.getEnchantment('fortune').level : 0;
         const hasSilkTouch = enchantments.hasEnchantment('silk_touch');
         const hammerBreakableBlocks = HAMMER_BLOCK_TYPES[tool.typeId];
         Durability(tool, player)
         if (hammerBreakableBlocks && hammerBreakableBlocks.includes(block.typeId)) {
            break3x3Area(block, player, hammerBreakableBlocks, fortuneLevel, hasSilkTouch);
         }
      }
   }catch(e){

   }
});

const w = world.getDimension("overworld");
world.afterEvents.worldInitialize.subscribe(start =>{
	w.runCommandAsync(`gamerule showtags false`);
})

world.afterEvents.entityHitBlock.subscribe( on =>{
		const p = on.damagingEntity;
		const equip = p.getComponent(EntityEquippableComponent.componentId);
		const item = equip.getEquipment(EquipmentSlot.Mainhand);
		const block = on.hitBlock
		if(item){
			if(item.typeId.includes("_drill") && block.typeId != "minecraft:bedrock"){
				const drillBreakableBlocks = DRILL_BLOCK_TYPES[item.typeId];
				Durability(item, p)
				if(!drillBreakableBlocks.includes(block.typeId)){
					p.runCommandAsync("playsound random.drill @s ~~~ 4 2 4")
					p.runCommandAsync(`fill ${block.x} ${block.y} ${block.z} ${block.x} ${block.y} ${block.z} air destroy`) 
				}  
			}
			else if(item.typeId.includes("_drill") && block.typeId == "minecraft:bedrock"){
				p.runCommandAsync("tellraw @s {\"rawtext\":[{\"translate\":\"action.drill.use\"}]}")
				p.runCommandAsync("playsound random.drill.bedrock @s ~~~ 4 1 4")
			}
		}
})

world.afterEvents.itemStopUse.subscribe(use =>{
	const item = use.itemStack;
	const p = use.source;
	if(item.typeId.includes("new:") && item.typeId.includes("_trident") && p.hasTag("use_trident")){
		custom_trident(p, item)
	}
	if(item.typeId.includes("new:") && item.typeId.includes("_bow") && p.hasTag("use_bow")){
		custom_bow(p, item)
	}
})


world.afterEvents.itemCompleteUse.subscribe(use =>{
	const item = use.itemStack;
	const p = use.source;
	if(item.typeId.includes("ambar_apple")){
		addEffect(p, "conduit_power", 1600, 3, true)
		addEffect(p, "jump_boost", 1600, 3, true)
		addEffect(p, "absorption", 1600, 2, true)
		addEffect(p, "strength", 1600, 3, true)
		addEffect(p, "health_boost", 1600, 5, true)
		addEffect(p, "resistance", 1600, 4, true)
		addEffect(p, "regeneration", 1600, 3, true)
	}
	if(item.typeId.includes("flourite_apple")){
		addEffect(p, "haste", 1100, 2, true)
		addEffect(p, "jump_boost", 1600, 3, true)
		addEffect(p, "night_vision", 1400, 3, true)
		addEffect(p, "resistance", 1300, 4, true)
		addEffect(p, "conduit_power", 800, 3, true)
	}
	if(item.typeId.includes("onix_apple")){
		addEffect(p, "absorption", 900, 2, true)
		addEffect(p, "water_breathing", 900, 5, true)
		addEffect(p, "haste", 900, 4, true)
		addEffect(p, "regeneration", 900, 3, true)
	}
	if(item.typeId.includes("ruby_apple")){
		addEffect(p, "invisibility", 1200, 3, true)
		addEffect(p, "fire_resistance", 1000, 5, true)
		addEffect(p, "absorption", 900, 3, true)
		addEffect(p, "resistance", 1100, 4, true)
		addEffect(p, "haste", 900, 1, true)
	}
	if(item.typeId.includes("sapphire_apple")){
		addEffect(p, "speed", 1200, 3, true)
		addEffect(p, "water_breathing", 1100, 5, true)
		addEffect(p, "jump_boost", 900, 2, true)
		addEffect(p, "resistance", 1200, 4, true)
		addEffect(p, "strength", 900, 1, true)
	}
	if(item.typeId.includes("topacio_apple")){
		addEffect(p, "village_hero", 900, 4, true)
		addEffect(p, "health_boost", 900, 5, true)
		addEffect(p, "strength", 900, 2, true)
	}
	if(item.typeId.includes("potion_ambar")){
		addEffect(p, "absorption", 1500, 0, true)
		addEffect(p, "haste", 1800, 0, true)
		addEffect(p, "strength", 900, 0, true)
		addEffect(p, "resistance", 900, 0, true)
		addEffect(p, "speed", 900, 0, true)
	}
	if(item.typeId.includes("potion_platino")){
		addEffect(p, "jump_boost", 1200, 0, true)
		addEffect(p, "resistance", 1200, 0, true)
		addEffect(p, "haste", 1200, 0, true)
	}
	if(item.typeId.includes("potion_ruby")){
		addEffect(p, "fire_resistance", 1200, 0, true)
		addEffect(p, "resistance", 900, 0, true)
		addEffect(p, "absorption", 1500, 0, true)
	}
	if(item.typeId.includes("potion_sapphire")){
		addEffect(p, "conduit_power", 1500, 0, true)
		addEffect(p, "speed", 900, 0, true)
		addEffect(p, "night_vision", 1200, 0, true)
	}
	if(item.typeId.includes("potion_jade")){
		addEffect(p, "jump_boost", 900, 0, true)
		addEffect(p, "village_hero", 1800, 0, true)
		addEffect(p, "speed", 1200, 0, true)
	}
	if(item.typeId.includes("potion_onix")){
		addEffect(p, "bad_omen", 2100, 0, true)
		addEffect(p, "resistance", 1200, 0, true)
		addEffect(p, "invisibility", 1500, 0, true)
	}
})


world.afterEvents.itemUse.subscribe(use =>{
	const item = use.itemStack;
	const p = use.source;
	if(item.typeId.includes("new:") && item.typeId.includes("_trident") && !p.hasTag("use_trident") && !p.hasTag("cooldown")){
		p.addTag("use_trident")
	}
	if(item.typeId.includes("new:") && item.typeId.includes("_bow") && !p.hasTag("use_bow")){
		const inv = p.getComponent("inventory").container
		const arrowItem = item.typeId.replace("bow", "arrow")
		for(let slot = 0; slot < inv.size; slot++){
			const itemInv = inv.getItem(slot);
			if(itemInv){
				if(itemInv.typeId == `${arrowItem}`){
					p.addTag("use_bow")
				}
			}
		}
	}
})

const trident_type = ["ambar", "uranium", "rodio", "topacio", "ruby", "sapphire", "platino", "flourita", "jade", "onix"]

system.runInterval(() =>{
	try{
		for (const p of world.getPlayers()) {
			p.runCommandAsync(`function gamerule`)
			p.runCommandAsync(`function rings_function`)
			shields_loop(p)
            detectEnchantements(p)
			item_animations(p)
			for(const type of trident_type){
				const trident = w.getEntities({type: `new:${type}_t`})
				for(const e of trident){
					if(e.hasTag("sensor")){
						e.getTags().forEach(t=>{
							if(t.startsWith('{"itemTag":{')){
								const itemId = JSON.parse(t)['itemTag']['id']
								const itemDag = JSON.parse(t)['itemTag']['damage']
								const drop_item = e.dimension.spawnEntity("new:trident_equip", {x: e.location.x, y: e.location.y, z: e.location.z})
								drop_item.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 ${itemId} 1 ${itemDag}`)
								e.runCommandAsync("event entity @s new:item_spawn")
								e.removeTag(t)
								system.runTimeout(()=>{
									drop_item.runCommandAsync("event entity @s drop_equip")
								}, 1)
							}
						})
					}
				}
			}
		}
	}catch(e){

	}
})

world.afterEvents.entityDie.subscribe(dead =>{
	try{
		const p = dead.deadEntity;
		if(p.typeId == "minecraft:player"){
			setScore(p, "ambar_totem", 0)
		}
	}catch(e){}
})

world.afterEvents.entityHurt.subscribe( hurt =>{
	try{
		const eHurt = hurt.hurtEntity;
		const damaging = hurt.damageSource.damagingEntity;
		const cause = hurt.damageSource.cause;
		if(eHurt&&eHurt.typeId == "minecraft:player"){
			const equip = eHurt.getComponent(EntityEquippableComponent.componentId);
			const hand = equip.getEquipment(EquipmentSlot.Mainhand);
			const offhand = equip.getEquipment(EquipmentSlot.Offhand);
			if(cause == "entityAttack" || cause == "entityExplosion" || cause == "projectile"){
				if(hand){
					if(hand.typeId == "new:ambar_totem"){
						ambar_totem(eHurt)
					}
				}
				if(offhand){
					if(offhand.typeId == "new:ambar_totem"){
						ambar_totem(eHurt)
					}
				}
			}
		}
	}
	catch(e){
	}
})

world.afterEvents.projectileHitEntity.subscribe(hit=>{
	try{
		const p = hit.source;
		const projectile = hit.projectile;
		const target = hit.getEntityHit()
		if(projectile && target){
			for(const bows of tags){
				if(projectile.typeId.includes(`new:${bows}_arrow`)){
					if(p.hasTag(`enchanted:infinity`)){
						projectile.remove()
					}
				}
			}
		}
	}catch(e){

	}
})

world.afterEvents.projectileHitBlock.subscribe(hit=>{
	try{
		const p = hit.source;
		const projectile = hit.projectile;
		const target = hit.getBlockHit()
		if(projectile && target){
			for(const bows of tags){
				if(projectile.typeId.includes(`new:${bows}_arrow`)){
					if(p.hasTag(`enchanted:infinity`)){
						projectile.remove()
					}
				}
			}
		}
	}catch(e){

	}
})

function detectEnchantements(p){
	const equip = p.getComponent(EntityEquippableComponent.componentId);
	const hand = equip.getEquipment(EquipmentSlot.Mainhand);
	if(hand){ 
		const enchantable = hand.getComponent("enchantable");
		for(const bows of tags){
			if(hand.typeId.includes(`${bows}_bow`)){
				if (enchantable) {
					if (enchantable.getEnchantment("infinity")) {
						p.addTag(`enchanted:infinity`)
					}
					if (!enchantable.getEnchantment("infinity")) {
						p.removeTag(`enchanted:infinity`)
					}
					if(enchantable.getEnchantments().length === 0){
						p.getTags().forEach(tags => {
							if(tags.includes("enchanted:")){
								p.removeTag(tags)
							}
						});
					}
				}
			}
		}
		if(!hand.typeId.includes(`_bow`)){
			p.getTags().forEach(tag => {
				if(tag.includes("enchanted:")){
					p.removeTag(tag)
				}
			});
		}
	}
	if (!hand) {
		p.getTags().forEach(tag => {
			if(tag.includes("enchanted:")){
				p.removeTag(tag)
			}
		});
	}
}
